@@ -2,7 +2,8 @@ import logging
 import os
 import shlex
 import subprocess
-from typing import Collection, Tuple, Union
+import sys
+from typing import Callable, Collection, Optional, Tuple, Union
 
 logger = logging.getLogger(__name__)
 
@@ -12,7 +13,7 @@ DIFF_EXCLUDE = [
 ]
 
 
-def _run_command(command: Union[str, Collection]) -> Tuple[int, bytes]:
+def _run_command(command: Union[str, Collection], output_callback=None) -> Tuple[int, bytes]:
     """Run a command in a shell.
 
     Args:
@@ -26,9 +27,18 @@ def _run_command(command: Union[str, Collection]) -> Tuple[int, bytes]:
         command = shlex.join(command)
     logger.debug("Run command: {}".format(command))
     p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-    output = p.stdout.read()
-    p.wait()
-    logger.debug("Command output: {}".format(output))
+    output = b""
+    if output_callback:
+        while True:
+            _line = p.stdout.readline()
+            if not _line:
+                break
+            output += _line
+            output_callback(_line)
+    else:
+        output = p.stdout.read()
+        p.wait()
+        logger.debug("Command output: {}".format(output))
     return p.returncode, output
 
 
@@ -87,8 +97,8 @@ def get_change_files(target, exclude_files=None):
     """
     if exclude_files is None:
         exclude_files = DIFF_EXCLUDE
-    exclude_files_args = " ".join(['":(exclude){}"'.format(f) for f in exclude_files])
-    cmd = "git diff --name-only {} {}".format(target, exclude_files_args)
+    exclude_files_args = " ".join([":(exclude){}".format(f) for f in exclude_files])
+    cmd = ["git", "--no-pager", "diff", "--cached", "--name-only", target, exclude_files_args]
     res, output = _run_command(cmd)
     if res != 0:
         logging.error("git command failed, cmd: {}".format(cmd))
@@ -112,12 +122,15 @@ def get_file_diff(path, target):
     def _join_path(_path):
         logger.debug("path: {}".format(_path))
         if isinstance(_path, list):
-            return " ".join([_join_path(p) for p in _path])
+            return [_join_path(p) for p in _path]
         if current != git_root:
             _path = os.path.join(git_root, _path).split(current + "/")[1]
         return _path
 
-    cmd = "git diff {} {}".format(target, _join_path(path))
+    cmd = ["git", "--no-pager", "diff", "--cached", target]
+    _path = _join_path(path)
+    if _path:
+        cmd.extend([_path] if isinstance(_path, str) else _path)
     res, output = _run_command(cmd)
     if res != 0:
         logging.error("git command failed, cmd: {}".format(cmd))
@@ -131,14 +144,8 @@ def commit(message):
     Args:
         message (str): The commit message.
 
-    Returns:
-        bool: True if the commit succeed, False otherwise.
     """
-    res, output = _run_command(["git", "commit", "-m", message])
-    if res != 0:
-        logging.error("git command failed")
-        return False
-    return True
+    _run_command(["git", "commit", "-m", message], lambda x: sys.stdout.write(x.decode("utf-8")))
 
 
 if __name__ == "__main__":
